#error DO NOT INCLUDE - DOCUMENTATION PURPOSE ONLY

//==================================================================================================
//! \mainpage The C++ Information Broker
//!
//! Detection and versioning of operating systems, compilers, architecture and other element are
//! traditionally done using preprocessor macros. Library like
//! [Boost.Predef](https://www.boost.org/doc/libs/release/doc/html/predef.html) provides a sanitized
//! interface on top of those but still live in a world where the preprocessor is king.
//!
//! SPY is a C++17 (and onward) library that gathers similar information and provides a `constexpr`
//! compatible interface to access those information, thus making their exploitation within `constexpr`
//! context possible.
//!
//! # Some References
//!
//! - [Our CppCon 2019 Lightning talk slides](https://docs.google.com/presentation/d/1nSBhU4pr5EWznni0MYsyDkMCr3O3q2XS-KQdz2_BRRI/edit?usp=sharing)
//! - [Our CppCon 2019 Lightning talk:](https://www.youtube.com/watch?v=t406o2EhG-A)
//!   @htmlonly <center><iframe width="560" height="315" src="https://www.youtube.com/embed/t406o2EhG-A" title="I SPY with my little eye Boost.Predef in a constexpr world - Joel Falcou - CppCon 2019" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture; web-share" allowfullscreen></iframe></center> @endhtmlonly
//!
//! # A Short Example
//!
//! @code
//! #include <iostream>
//! #include <spy/spy.hpp>
//!
//! int main()
//! {
//!   std::cout << "OS                : " << spy::operating_system      << std::endl;
//!   std::cout << "CPU Architecture  : " << spy::architecture          << std::endl;
//!   std::cout << "SIMD Instructions : " << spy::simd_instruction_set  << std::endl;
//!   std::cout << "Compiler          : " << spy::compiler              << std::endl;
//!   std::cout << "libc version      : " << spy::libc                  << std::endl;
//!   std::cout << "libstd            : " << spy::stdlib                << std::endl;
//! }
//! @endcode
//!
//! # Licence
//!
//! This library is licensed under the [Boost Software License](https://opensource.org/licenses/BSL-1.0):
//!
//! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//! Copyright : SPY Project Contributors
//!
//! Permission is hereby granted, free of charge, to any person or organization obtaining a copy of the
//! software and accompanying documentation covered by this license (the "Software") to use, reproduce,
//! display, distribute, execute, and transmit the Software, and to prepare derivative works of the
//! Software, and to permit third-parties to whom the Software is furnished to do so, all subject to the
//! following:
//!
//! The copyright notices in the Software and this entire statement, including the above license grant,
//! this restriction and the following disclaimer, must be included in all copies of the Software, in
//! whole or in part, and all derivative works of the Software, unless such copies or derivative works
//! are solely in the form of machine-executable object code generated by a source language processor.
//!
//! THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
//! NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
//! NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE
//! BE LIABLE FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING
//! FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//!
//==================================================================================================
